function slabel = runNASNet(filename,gamma,net_name,sort_labels)
% This script classifies waveforms using a trained neural
% network (see Issar et al (2020)). The file can either be an NEV
% or a .mat file containing a variable named 'waveforms'.
%INPUTS:
%       filename- string filename of a .nev file OR a .mat file.
%                 IF it's a .mat file, MUST include a variable called "waveforms",
%                 waveforms is an Nxn array, where N is the number
%                 of waveforms, and n is the number of samples in
%                 each waveform* (see NOTES below)
%       gamma- minimum P(spike) value for a waveform to be classified as a spike (between 0 and 1).
%              If you want a more lenient sort (i.e. allows for more noise), then choose a smaller gamma.
%       net_name- string of network name. All four NASNet output
%                files must be available and saved as the network name followed by 
%                _w_hidden, _w_output, _b_hidden, _b_output.
%       sort_labels- 2x1 integer array of sortcodes (must be an integer between 0 and 255). The first 
%                    value is the label that all spikes will be assigned.The second value is the 
%                    label that all noise will be assigned.  
%
%OUTPUTS:
%       slabel- list of labels for each waveform 
%               (ex. if the sort_labels input was [1,0], then the output list will label spikes 1 and
%               noise 0).

%NOTES:
%****If you load an NEV file, this script will modify its sort codes.
%****The number of samples in each waveform must match the number of
%    samples in the waveforms the network was trained on. The Issar et al
%    (2020) network was trained on waveforms with 52 samples.
%% load trained network
try
    w1 = load(strcat(net_name,'_w_hidden'));
    b1 = load(strcat(net_name,'_b_hidden'));
    w2 = load(strcat(net_name,'_w_output'));
    b2 = load(strcat(net_name,'_b_output'));
catch
    error('The network you named does not exist or the files were not named appropriately.')
end

%% load file to classify
[~,~,ext] = fileparts(filename);

switch ext
    case '.nev'
        [spikes,waves] = read_nev(filename);
        n_samples = size(waves{find(spikes(:,1)~=0,1)},1); % n, number of samples in a waveform
        if any(spikes(:,1)==0)
            %These are digital codes. There are no waveforms to classify for
            %these indices so create placeholder in waveform list so indexing works
            %for moveToSortCode.
            waves(spikes(:,1)==0) = {ones(n_samples,1,'int16')};
        end
        
        waveforms = [waves{:}]'; %convert from a cell to an array
        clear waves;
        
    case '.mat'
        load(filename);
        if ~exist('waveforms'), error('waveforms must be stored in a variable named "waveforms".'); end
        n_samples = size(waveforms,2);
        
    otherwise
        error('file must be a .nev or .mat');
end

%check that the waveform size is appropriate for the network
if n_samples~=size(w1,1), error(['The network called ', net_name,' can only classify waveforms that are ' num2str(size(w1,1)),' samples long']); end

%check that sort_labels are integers between 0 and 255
assert(unique(ismember(sort_labels(1),0:255)),'The spike sort_label must be 0 to 255');
assert(unique(ismember(sort_labels(2),0:255)),'The noise sort_label must be 0 to 255');

%% classify waveforms
nwaves = length(waveforms);
n_per_it = 1000; %number of waves per iteration
counter = 0;
nend = 0;

net_labels = zeros(1,nwaves);

while nend~=nwaves
    nstart = 1 + counter*n_per_it;
    nend = nstart + n_per_it - 1;
    
    if nend>nwaves %at the end of the file
        nend = nwaves;
    end
    
    n_loop = length(nstart:nend);
    
    % generate classifications
    wave_in = double(waveforms(nstart:nend,:));
    
    %*****Layer 1******
    layer1_raw = wave_in*w1 + repmat(b1',n_loop,1);
    
    %***ReLU activation****
    %layer1_out = log(1+exp(layer1_raw)); %faster ReLU
    layer1_out = max(0,layer1_raw); %more accurate ReLU
    
    %***Layer 2*******
    layer2_raw = layer1_out*w2 + repmat(b2',n_loop,1);
    
    %***Sigmoid******
    layer2_out = 1./(1+exp(-1*(layer2_raw))); %apply sigmoid
    
    net_labels(nstart:nend) = layer2_out;
    counter = counter + 1;
end

%% assign waveforms spike/noise labels
%list of logicals (spike-1 or noise-0)
slabel = zeros(size(net_labels)); 
slabel(net_labels>=gamma) = 1; %spike classification

%If applicable, modify NEV file
if strcmp(ext,'.nev')
    %spike labels
    slabel(spikes(:,1)==0) = 0; %set logical to 0 for digital codes so we don't change them
    moveToSortCode(filename,slabel,sort_labels(1)); %assign spikes new label
    
    %noise labels
    nlabel = ~slabel;
    nlabel(spikes(:,1)==0) = 0; %again, don't touch digital codes
    
    moveToSortCode(filename, nlabel, sort_labels(2)); %assign noise new label
end

%modify output labels to match sort_labels
slabel(slabel==1) = sort_labels(1); 
slabel(slabel==0) = sort_labels(2); 

